@@ -318,7 +318,7 @@ UFSSwapDir::statfs(StoreEntry & sentry) const
     int x;
     storeAppendPrintf(&sentry, "First level subdirectories: %d\n", l1);
     storeAppendPrintf(&sentry, "Second level subdirectories: %d\n", l2);
-    storeAppendPrintf(&sentry, "Maximum Size: %"PRIu64" KB\n", maxSize() >> 10);
+    storeAppendPrintf(&sentry, "Maximum Size: %" PRIu64 " KB\n", maxSize() >> 10);
     storeAppendPrintf(&sentry, "Current Size: %.2f KB\n", currentSize() / 1024.0);
     storeAppendPrintf(&sentry, "Percent Used: %0.2f%%\n",
                       Math::doublePercent(currentSize(), maxSize()));
@@ -395,7 +395,7 @@ UFSSwapDir::maintain()
         if (!e)
             break;		/* no more objects */
 
-        removed++;
+        ++removed;
 
         e->release();
     }
@@ -557,7 +557,7 @@ UFSSwapDir::verifyCacheDirs()
     if (!pathIsDirectory(path))
         return true;
 
-    for (int j = 0; j < l1; j++) {
+    for (int j = 0; j < l1; ++j) {
         char const *aPath = swapSubDir(j);
 
         if (!pathIsDirectory(aPath))
@@ -572,7 +572,7 @@ UFSSwapDir::createSwapSubDirs()
 {
     LOCAL_ARRAY(char, name, MAXPATHLEN);
 
-    for (int i = 0; i < l1; i++) {
+    for (int i = 0; i < l1; ++i) {
         snprintf(name, MAXPATHLEN, "%s/%02X", path, i);
 
         int should_exist;
@@ -584,7 +584,7 @@ UFSSwapDir::createSwapSubDirs()
 
         debugs(47, 1, "Making directories in " << name);
 
-        for (int k = 0; k < l2; k++) {
+        for (int k = 0; k < l2; ++k) {
             snprintf(name, MAXPATHLEN, "%s/%02X/%02X", path, i, k);
             createDirectory(name, should_exist);
         }
@@ -609,7 +609,7 @@ UFSSwapDir::logFile(char const *ext) const
         while (strlen(pathtmp) && pathtmp[strlen(pathtmp) - 1] == '.')
             pathtmp[strlen(pathtmp) - 1] = '\0';
 
-        for (pathtmp2 = pathtmp; *pathtmp2 == '.'; pathtmp2++);
+        for (pathtmp2 = pathtmp; *pathtmp2 == '.'; ++pathtmp2);
         snprintf(lpath, MAXPATHLEN - 64, Config.Log.swap, pathtmp2);
 
         if (strncmp(lpath, Config.Log.swap, MAXPATHLEN - 64) == 0) {
@@ -728,6 +728,19 @@ UFSSwapDir::addDiskRestore(const cache_key * key,
     return e;
 }
 
+void
+UFSSwapDir::undoAddDiskRestore(StoreEntry *e)
+{
+    debugs(47, 5, HERE << *e);
+    replacementRemove(e); // checks swap_dirn so do it before we invalidate it
+    // Do not unlink the file as it might be used by a subsequent entry.
+    mapBitReset(e->swap_filen);
+    e->swap_filen = -1;
+    e->swap_dirn = -1;
+    cur_size -= fs.blksize * sizeInBlocks(e->swap_file_sz);
+    --n_disk_objects;
+}
+
 void
 UFSSwapDir::rebuild()
 {
@@ -804,9 +817,8 @@ UFSSwapDir::openTmpSwapLog(int *clean_flag, int *zero_flag)
         buf.init(header.record_size, header.record_size);
         buf.append(reinterpret_cast<const char*>(&header), sizeof(header));
         // Pad to keep in sync with UFSSwapDir::writeCleanStart().
-        // TODO: When MemBuf::spaceSize() is fixed not to subtract one,
-        // memset() space() with zeroes and use spaceSize() below.
-        buf.appended(static_cast<size_t>(header.record_size) - sizeof(header));
+        memset(buf.space(), 0, header.gapSize());
+        buf.appended(header.gapSize());
         file_write(swaplog_fd, -1, buf.content(), buf.contentSize(),
                    NULL, NULL, buf.freeFunc());
     }
@@ -895,6 +907,7 @@ UFSSwapDir::writeCleanStart()
     /*copy the header */
     memcpy(state->outbuf, &header, sizeof(StoreSwapLogHeader));
     // Leave a gap to keep in sync with UFSSwapDir::openTmpSwapLog().
+    memset(state->outbuf + sizeof(StoreSwapLogHeader), 0, header.gapSize());
     state->outbuf_offset += header.record_size;
 
     state->walker = repl->WalkInit(repl);
@@ -944,6 +957,7 @@ UFSCleanLog::write(StoreEntry const &e)
     s.refcount = e.refcount;
     s.flags = e.flags;
     memcpy(&s.key, e.key, SQUID_MD5_DIGEST_LENGTH);
+    s.finalize();
     memcpy(outbuf + outbuf_offset, &s, ss);
     outbuf_offset += ss;
     /* buffered write */
@@ -1052,6 +1066,7 @@ UFSSwapDir::logEntry(const StoreEntry & e, int op) const
     s->refcount = e.refcount;
     s->flags = e.flags;
     memcpy(s->key, e.key, SQUID_MD5_DIGEST_LENGTH);
+    s->finalize();
     file_write(swaplog_fd,
                -1,
                s,
@@ -1130,7 +1145,8 @@ UFSSwapDir::DirClean(int swap_index)
                 if (UFSSwapDir::FilenoBelongsHere(fn, D0, D1, D2))
                     continue;
 
-        files[k++] = swapfileno;
+        files[k] = swapfileno;
+        ++k;
     }
 
     closedir(dir_pointer);
@@ -1143,7 +1159,7 @@ UFSSwapDir::DirClean(int swap_index)
     if (k > 10)
         k = 10;
 
-    for (n = 0; n < k; n++) {
+    for (n = 0; n < k; ++n) {
         debugs(36, 3, "storeDirClean: Cleaning file "<< std::setfill('0') << std::hex << std::uppercase << std::setw(8) << files[n]);
         snprintf(p2, MAXPATHLEN + 1, "%s/%08X", p1, files[n]);
         safeunlink(p2, 0);
@@ -1175,7 +1191,7 @@ UFSSwapDir::CleanEvent(void *unused)
          */
         UFSDirToGlobalDirMapping = (int *)xcalloc(NumberOfUFSDirs, sizeof(*UFSDirToGlobalDirMapping));
 
-        for (i = 0, n = 0; i < Config.cacheSwap.n_configured; i++) {
+        for (i = 0, n = 0; i < Config.cacheSwap.n_configured; ++i) {
             /* This is bogus, the controller should just clean each instance once */
             sd = dynamic_cast <SwapDir *>(INDEXSD(i));
 
@@ -1186,7 +1202,8 @@ UFSSwapDir::CleanEvent(void *unused)
 
             assert (usd);
 
-            UFSDirToGlobalDirMapping[n++] = i;
+            UFSDirToGlobalDirMapping[n] = i;
+            ++n;
 
             j += (usd->l1 * usd->l2);
         }
@@ -1203,7 +1220,7 @@ UFSSwapDir::CleanEvent(void *unused)
     /* if the rebuild is finished, start cleaning directories. */
     if (0 == StoreController::store_dirs_rebuilding) {
         n = DirClean(swap_index);
-        swap_index++;
+        ++swap_index;
     }
 
     eventAdd("storeDirClean", CleanEvent, NULL,
@@ -1303,7 +1320,7 @@ UFSSwapDir::unlink(StoreEntry & e)
 {
     debugs(79, 3, "storeUfsUnlink: dirno " << index  << ", fileno "<<
            std::setfill('0') << std::hex << std::uppercase << std::setw(8) << e.swap_filen);
-    if (e.swap_status == SWAPOUT_DONE && EBIT_TEST(e.flags, ENTRY_VALIDATED)) {
+    if (e.swap_status == SWAPOUT_DONE) {
         cur_size -= fs.blksize * sizeInBlocks(e.swap_file_sz);
         --n_disk_objects;
     }
@@ -1345,7 +1362,7 @@ UFSSwapDir::replacementRemove(StoreEntry * e)
 void
 UFSSwapDir::dump(StoreEntry & entry) const
 {
-    storeAppendPrintf(&entry, " %"PRIu64" %d %d", maxSize() >> 20, l1, l2);
+    storeAppendPrintf(&entry, " %" PRIu64 " %d %d", maxSize() >> 20, l1, l2);
     dumpOptions(&entry);
 }
 
